use Config;
use File::Basename qw(basename dirname);
chdir(dirname($0));

$perl = shift(@ARGV);

$use = "use lib qw(\n\t" . join("\n\t", @ARGV) . ");\n";

($file = basename($0)) =~ s/\.PL$//;
$file =~ s/\.pl$//
	if ($Config{'osname'} eq 'VMS' or
	    $Config{'osname'} eq 'OS2');  # "case-forgiving"
open OUT,">$file" or die "Can't create $file: $!";
chmod(0755, $file);
print "Extracting $file (with variable substitutions)\n";

print OUT <<"!GROK!THIS!";
#!$perl

#
# Warning: File automatically generated. Any changes made here
# will be lost.
#

$use

!GROK!THIS!
print OUT <<'!NO!SUBS!';
=head1 NAME

gen-catalog

=head1 SYNOPSIS

gen-catalog repository-name

=head1 DESCRIPTION

Create the catalog entries for this repository. This script uses the
PageTemplate object to load a template for a page from a file. Templates
are placed by default in installdir/lib/templates/template-name.htm.

=cut

use strict;
use vars qw(%Domains);

use MTk;
require PageTemplate;

#
# For PageTemplate...
#
$::TEMPLATE_DIR = "$MTk::install_lib/templates";

#
# Initialize the domain list. This list was created from
# http://nhse.cs.utk.edu/sw_catalog/HPCC_domains.txt via
# the compile_domains script.
#

$Domains{'top'} = new Domain('top');
$Domains{'application'} =
    new Domain('application',
	       $Domains{'top'});
$Domains{'benchmark'} =
    new Domain('benchmark',
	       $Domains{'top'});
$Domains{'data analysis visualization'} =
    new Domain('data analysis visualization',
	       $Domains{'top'});
$Domains{'distributed processing tool'} =
    new Domain('distributed processing tool',
	       $Domains{'top'});
$Domains{'educational'} =
    new Domain('educational',
	       $Domains{'top'});
$Domains{'numerical'} =
    new Domain('numerical',
	       $Domains{'top'});
$Domains{'numerical/computational geometry'} =
    new Domain('computational geometry',
	       $Domains{'numerical'});
$Domains{'numerical/linear algebra'} =
    new Domain('linear algebra',
	       $Domains{'numerical'});
$Domains{'numerical/optimization'} =
    new Domain('optimization',
	       $Domains{'numerical'});
$Domains{'numerical/PDEs'} =
    new Domain('PDEs',
	       $Domains{'numerical'});
$Domains{'numerical/miscellaneous'} =
    new Domain('miscellaneous',
	       $Domains{'numerical'});
$Domains{'parallel processing tool'} =
    new Domain('parallel processing tool',
	       $Domains{'top'});
$Domains{'parallel processing tool/communication library'} =
    new Domain('communication library',
	       $Domains{'parallel processing tool'});
$Domains{'parallel processing tool/execution performance analyzer'} =
    new Domain('execution performance analyzer',
	       $Domains{'parallel processing tool'});
$Domains{'parallel processing tool/parallel programming environment'} =
    new Domain('parallel programming environment',
	       $Domains{'parallel processing tool'});
$Domains{'parallel processing tool/parallel I/O system'} =
    new Domain('parallel I/O system',
	       $Domains{'parallel processing tool'});
$Domains{'parallel processing tool/parallel programming language compiler'} =
    new Domain('parallel programming language compiler',
	       $Domains{'parallel processing tool'});
$Domains{'parallel processing tool/parallel programming library'} =
    new Domain('parallel programming library',
	       $Domains{'parallel processing tool'});
$Domains{'parallel processing tool/parallel runtime system'} =
    new Domain('parallel runtime system',
	       $Domains{'parallel processing tool'});
$Domains{'parallel processing tool/source code analyzer restructurer'} =
    new Domain('source code analyzer restructurer',
	       $Domains{'parallel processing tool'});
$Domains{'parallel processing tool/miscellaneous'} =
    new Domain('miscellaneous',
	       $Domains{'parallel processing tool'});

postprocess_domains(\%Domains);


eval {
    &main;
};

if ($@)
{
    print "gen-catalog failed:\n\t$@\n";
    exit(1);
}


exit(0);

#
# Postprocess the domain list.
#
#    Set up the parent->child links.
#    Set up the path-based page anchor names.
# 
#
sub postprocess_domains
{
    my($domains) = @_;
    my($path,$dobj, $anchor);
    
    for $dobj (values(%$domains))
    {
	$dobj->setup_parent_link();
    }

    while (($path, $dobj) = each(%$domains))
    {
	($anchor = $path) =~ s/[^a-zA-Z_0-9]/_/g;
	$dobj->anchor($anchor);
    }
	
}

sub make_anchor
{
    my($text, $name) = @_;
    return "<a name=\"$name\">$text</a>";
}

sub make_link
{
    my($text, $target) = @_;
    return "<a href=\"$target\">$text</a>";
}

sub main
{
    my($db, $repos_name, $info);

    if (@ARGV == 1)
    {
	($repos_name) = (@ARGV);
    }
    else
    {
	die "Usage: $0 repository-name\n";
    }

    #
    # Info is a hash used for storing miscellaneous (sorta per-class style)
    # information.
    #

    $info = {
	output_dir => "/home/www/olson/tmp",
	output_url => "http://www.mcs.anl.gov/home/olson/tmp",
	page_idx => 0,
    };

    $db = $MTk::DBClass->new();
    $db or die "Cannot connect to database\n";
    
    $db->open();

    &validate_repository($db, $repos_name);
    &gen_catalog($db, $repos_name, $info);
}

#
# Make sure that this is a valid repository.
#

sub validate_repository
{
    my($db, $repos_name) = @_;
    my($q, $official_name);

    $q = $db->query("select official_name from Repository where name = '$repos_name'");
    $q or die "Repository $repos_name not found\n";

    $official_name = $q->item(0, 0);
    print "Listing assets in repository $official_name\n";
}


#
# Generate the catalog.
# 

sub gen_catalog
{
    my($db, $repos_name, $info) = @_;
    my($domains);
    my($toc, $blurb);

    $domains = \%Domains;

    load_assets($db, $repos_name, $domains, $info);

    $toc = $domains->{top}->gen_toc();
    $blurb = $domains->{top}->gen_blurbs();

    my($toc_file);
    $toc_file = "$info->{output_dir}/toc.html";
    open(F, ">$toc_file") or die "cannot open $toc_file: $!\n";
    print F $toc, "\n";
    print F $blurb, "\n";
    close(F);
}

#
# Obtain the list of assets
#
sub load_assets
{
    my($db, $repos_name, $domains, $info) = @_;
    my($q, $asset, $a);
    my($id, $repos, $exp, $last_retr);
    
    $q = $db->query("SELECT id FROM Asset where repository = '$repos_name'");

    my $count = 10;
    if ($q)
    {
	for $a (@{$q->row_lists})
	{
#	    last if $count-- < 0;
	    #
	    # Fetch asset, and parse to a BIDM object
	    #
	    
	    ($id) = @$a;
	    $asset = MTk::DBTable::Asset->new($id, $db);
	    $asset->fetch();
#	    $asset->print();
	    my $ts = new BIDM::Tagset;
	    $ts->load_from_db($db, $id);
#	    $ts->print();
	    my $obj = eval { $ts->parse(); };
	    next unless $obj;

	    #
	    # Deal with it...
	    #
	    if ($obj->isa("Asset"))
	    {
		&handle_asset($obj, $domains, $info);
	    }
	    elsif ($obj->isa("Library"))
	    {
		print "Found library ", $obj->Name(), "\n";
	    }
	}
    }
}
#
# 
#
sub handle_asset
{
    my($obj, $domains, $info) = @_;
    my($name, $domain, $dobj);

    $name = $obj->Name();
    $domain = $obj->Domain();

    print "Asset: name=$name domain=$domain\n";

    #
    # Normalize & look up the domain.
    #

    $domain =~ s/^\s+//;
    $domain =~ s/\s+$//;

    $dobj = $domains->{$domain};
    if ($dobj)
    {
#	print "Found domain $dobj->{name}\n";
	$dobj->add_asset($obj);
    }
    else
    {
	print "Domain for $domain not found\n";
    }

    create_asset_page($obj, $info);
}

sub simple_attribute
{
    my($asset, $body_ref, $name, $print_name) = @_;
    my($v);

    $print_name = $name if $print_name eq '';
    
    if (defined($v = $asset->$name()))
    {
	$$body_ref .= "<dt><strong>$print_name:</strong>\n";
	if (ref($v))
	{
	    my $url = $v->href();
	    $$body_ref .= "<dd>" . ::make_link($url, $url) . "\n";
	}
	else
	{
	    $$body_ref .= "<dd>$v\n";
	}
    }
}

sub multiple_attribute
{
    my($asset, $body_ref, $name, $print_name) = @_;
    my($v, @v);

    $print_name = $name if $print_name eq '';
    
    if (defined(@v = $asset->$name()))
    {
	$$body_ref .= "<dt><strong>$print_name:</strong>\n";

	for $v (@v)
	{
	    if (ref($v))
	    {
		my $url = $v->href();
		$$body_ref .= "<dd>" . ::make_link($url, $url) . "\n";
	    }
	    else
	    {
		$$body_ref .= "<dd>$v\n";
	    }
	}
    }
}

#
# Create the per-asset information page
#
sub create_asset_page
{
    my($asset, $info) = @_;
    my($page_url, $page_idx, $page_file);
    my($fh, $page, $body, $v, @v);
    
    $page_idx = $info->{page_idx}++;
    
    $page_file = "$info->{output_dir}/$page_idx.html";
    $page_url = "$info->{output_url}/$page_idx.html";

    $asset->user_hash()->{page_url} = $page_url;

    $fh = new FileHandle(">$page_file");
    if (!$fh)
    {
	die "Cannot open $page_file: $!\n";
    }

    $page = new PageTemplate('catalog_entry')->load();
    $page->replace("#PAGE_TITLE#", $asset->Name());
    $page->replace("#TITLE#", $asset->Name());

    $body = '';

    $body .= "<dl>\n";

    simple_attribute($asset, \$body, "WebPage", "URL");
    simple_attribute($asset, \$body, "Version");
    simple_attribute($asset, \$body, "Abstract");
    simple_attribute($asset, \$body, "Language");
    simple_attribute($asset, \$body, "TargetEnvironment", "Target environment");
    multiple_attribute($asset, \$body, "Keyword");
    simple_attribute($asset, \$body, "IsLocatedIn", "Repository");
    simple_attribute($asset, \$body, "Cost");
    if ((@v = $asset->ContactIs()) > 0)
    {
	$body .= "<dt><strong>Contacts:</strong>\n";
	my $c;
	for $c (@v)
	{
	    if (defined($v = $c->Name()))
	    {
		$body .= "<dd>$v\n";
	    }
	    if (defined($v = $c->Address()))
	    {
		$body .= "<dd>$v\n";
	    }
	    if (defined($v = $c->Email()))
	    {
		$body .= "<dd>" . ::make_link($v, "mailto:$v") . "\n";
	    }
	    if (defined($v = $c->Telephone()))
	    {
		$body .= "<dd>$v\n";
	    }
	    if (defined($v = $c->Fax()))
	    {
		$body .= "<dd>FAX $v\n";
	    }
	}
    }
    simple_attribute($asset, \$body, "Reference");
    $body .= "</dl>\n";
    $page->replace("#BODY#", $body);
    $page->print($fh);

    $fh->close();
}
    

#
# The Domain object.
#

package Domain;

sub new
{
    my($class, $name, $parent) = @_;
    my($self);

    $self = {
	name => $name,
	anchor => $name,
	parent => $parent,
	subdomains => {},
	assets => [],
    };
    bless($self, $class);
    return $self;
}

sub setup_parent_link
{
    my($self) = @_;

    if ($self->parent())
    {
	$self->parent()->setup_child_link($self);
    }
}

sub setup_child_link
{
    my($self, $child) = @_;
    $self->subdomains()->{$child->name()} = $child;
}

sub add_asset
{
    my($self, $asset) = @_;
    push(@{$self->assets()}, $asset);
}

#
# The table of contents for a domain is the domain's name, followed
# by a bullet-list of the subdomains' tables of contents.
# 
sub gen_toc
{
    my($self) = @_;
    my($sub, @subs, $asset);
    my($my_toc, $sub_toc);

    $my_toc = "<a href=\"#" . $self->anchor() . "\">$self->{name}</a>\n";
    @subs = sort(keys(%{$self->{subdomains}}));
    if (@subs)
    {
	$my_toc .= "<UL>\n";
	for $sub (@subs)
        {
	    $my_toc .= "<LI>\n";
	    $sub_toc = $self->{subdomains}->{$sub}->gen_toc();
	    $my_toc .= $sub_toc;
	}
	$my_toc .= "</UL>\n";
    }
    return $my_toc;
}

#
# The blurb for a domain a definition list, where the
# term is the domain name and the definition is the list of
# assets within the domain.
sub gen_blurbs
{
    my($self) = @_;
    my($blurb, @subs, $sub, $asset);

    $blurb = "<dl>\n";
    $blurb .= "<dt><strong>" . ::make_anchor($self->name(), $self->anchor());
    $blurb .= "</strong>\n";
    @subs = sort($self->subdomain_names());

    for $sub (@subs)
    {
	my $sub_blurb = $self->subdomain($sub)->gen_blurbs();
	$blurb .= $sub_blurb;
    }

    #
    # Now the assets
    #
    for $asset ($self->sorted_assets())
    {
	my $asset_blurb = "<dd>";
	$asset_blurb .= ::make_link($asset->Name(),
				    $asset->user_hash()->{page_url});
	my $title = $asset->TitleLine();
	if (defined($title))
	{
	    $asset_blurb .= ", $title";
	}
	$blurb .= $asset_blurb;
    }
    $blurb .= "</dl>\n";
    return $blurb;
}

sub sorted_assets
{
    my($self) = @_;
    return sort { ($a->Name())[0] cmp ($b->Name())[0] } @{$self->{assets}};
}

sub subdomain
{
    my($self, $subname) = @_;
    return $self->{subdomains}->{$subname};
}

sub subdomain_names
{
    my($self) = @_;
    return keys(%{$self->{subdomains}});
}

sub write
{
    my($self) = @_;
    my($sub, @subs, $asset);

    print "$self->{name}\n";
    for $asset (sort { ($a->Name())[0] cmp ($b->Name())[0] } @{$self->{assets}})
    {
	print "<LI> $asset ", ($asset->Name())[0], "\n";
    }
    @subs = sort(keys(%{$self->{subdomains}}));
    if (@subs)
    {
	print "<UL>\n";
	for $sub (@subs)
        {
	    print "<LI> ";
	    $self->{subdomains}->{$sub}->write();
	}
	print "</UL>\n";
    }
}

sub name { shift->_elem('name', @_); }
sub parent { shift->_elem('parent', @_); }
sub subdomains { shift->_elem('subdomains', @_); }
sub assets { shift->_elem('assets', @_); }
sub anchor { shift->_elem('anchor', @_); }

sub _elem
{
    my($self, $elem, $val) = @_;
    my $old = $self->{$elem};
    $self->{$elem} = $val if defined $val;
    return $old;
}
!NO!SUBS!
