use Config;
use File::Basename qw(basename dirname);
chdir(dirname($0));

$perl = shift(@ARGV);

$use = "use lib qw(\n\t" . join("\n\t", @ARGV) . ");\n";

($file = basename($0)) =~ s/\.PL$//;
$file =~ s/\.pl$//
	if ($Config{'osname'} eq 'VMS' or
	    $Config{'osname'} eq 'OS2');  # "case-forgiving"
open OUT,">$file" or die "Can't create $file: $!";
chmod(0755, $file);
print "Extracting $file (with variable substitutions)\n";

print OUT <<"!GROK!THIS!";
#!$perl -w

#
# Warning: File automatically generated. Any changes made here
# will be lost.
#

$use

!GROK!THIS!
print OUT <<'!NO!SUBS!';

=head1 NAME

retrieve-metadata

=head1 SYNOPSIS

retrieve-metadata repository-name

=head1 DESCRIPTION

See http://nhse.mcs.anl.gov/NHSE96/RIB/MTk/tools/retrieve-metadata.htm
for a full description of this command.

=cut

use strict;

use BIDM;
use MTk;
use LWP::RobotUA;
use HTTP::Date;
#use LWP::Debug qw(+trace +debug +conns);

#eval {
    &main;
#};

if ($@)
{
    print "$0 failed:\n\t$@\n";
    exit(1);
}


exit(0);

sub main
{
    my($db, $repos_name, @assets, $asset, $user_agent, $bidm_tagset,
      $response);

    if (@ARGV == 1)
    {
	($repos_name) = (@ARGV);
    }
    else
    {
	die "Usage: $0 repository-name\n";
    }

    $db = $MTk::DBClass->new();
    $db or die "Cannot connect to database\n";
    
    $db->open();

    &validate_repository($db, $repos_name);
    @assets = &prepare_asset_list($db, $repos_name);

    $user_agent = new LWP::RobotUA("RIB/0.1", "olson\@mcs.anl.gov");

    #
    # For debugging, set the delay to zero. For production, set to
    # something else.
    #
    $user_agent->delay(0);

    for $asset (@assets)
    {
	$asset->fetch();
	$asset->print();

	($bidm_tagset, $response) = $asset->retrieve_metadata($user_agent);
#	print "Got tagset $bidm_tagset\n";
#	print "Last modification date: ",
# 		time2str($bidm_tagset->response->last_modified()), "\n";
# 	print "Retrieval date: ",
# 		time2str($bidm_tagset->response->date()), "\n";
#	$bidm_tagset->print();

	if (!$bidm_tagset)
	{
	    print "Hm, retrieval failed\n";
	    if ($response->code() == 304)
	    {
		print "Document not changed\n";
	    }
	    else
	    {
		print "Document retrieval error: ", $response->code(), ": ", $response->message(), "\n";
	    }
	}
	else
	{
	    $asset->store_tagset($bidm_tagset);
	}
    }

    print "exiting\n";
}

#
# Make sure that this is a valid repository.
#

sub validate_repository
{
    my($db, $repos_name) = @_;
    my($q, $official_name);

    $q = $db->query("select official_name from Repository where name = '$repos_name'");
    $q or die "Repository $repos_name not found\n";

    $official_name = $q->item(0, 0);
}


#
# Get the list of assets that need to be updated.
# 

sub prepare_asset_list
{
    my($db, $repos_name) = @_;
    my($q, $asset, $a);
    my($id, $repos, $exp, $last_retr);
    my($now, $expiry_time);
    my(@assets);

    $now = time;
    $expiry_time = $now;
#    $expiry_time = $now - 14 * 24 * 60 * 60;		 # Two weeks for now

    $q = $db->query(<<END);
	SELECT id FROM Asset
	WHERE
	    repository = '$repos_name' AND
	    last_retrieval = -1 OR
	    last_retrieval < $expiry_time OR
	    expires > $now
END

    if ($q)
    {
	for $a (@{$q->row_lists})
	{
	    ($id) = @$a;
	    push(@assets, MTk::DBTable::Asset->new($id, $db));
	}
    }
    return @assets;
}
!NO!SUBS!
