package BIDM::RIGObject::Asset;

=head1 NAME

BIDM::RIGObject::Asset

=head1 DESCRIPTION

=cut

use strict;

use BIDM;
use BIDM::RIGObject;
use BIDM::ClassDefinition;

use vars qw(@ISA);

@ISA = qw(BIDM::RIGObject);

my $ClassName = "Asset";
my $ClassDefinition = new BIDM::ClassDefinition($ClassName);

sub new
{
    my($class) = @_;
    my($self);

    $self = new BIDM::RIGObject();
    bless($self, $class);

    $self->class_name($ClassName);
    $self->class_definition($ClassDefinition);

    return $self;

}

sub map_attribute_name
{
    my($self, $name) = @_;
    my($cname);

#     if (!defined($MapsDefined))
#     {
# 	&_define_maps();
#     }

#    $cname = $CanonicalAttribute{"\L$name"};

    if (defined($cname))
    {
	return $cname;
    }
    else
    {
	return $self->BIDM::RIGObject::map_attribute_name($name);
    }
}

sub map_relationship_name
{
    my($self, $name) = @_;
    my($cname);

#     if (!defined($MapsDefined))
#     {
# 	&_define_maps();
#     }

#    $cname = $CanonicalRelationship{"\L$name"};

    if (defined($cname))
    {
	return $cname;
    }
    else
    {
	return $self->BIDM::RIGObject::map_relationship_name($name);
    }
}

1;
__END__

#
# Create the lower-case-to-canonical-name maps for the attributes
# and relationships of this object.
#
sub _define_maps
{
    my($name, $cname);

    $MapsDefined = 1;
    
    for $cname (@Attributes)
    {
	$name = "\L$cname";
	$CanonicalAttribute{$name} = $cname;
    }
    
    for $cname (@Relationships)
    {
	$name = "\L$cname";
	$CanonicalRelationship{$name} = $cname;
    }
    
}

sub Abstract { shift->_elem('Abstract', @_); }
sub Cost { shift->_elem('Cost', @_); }
sub DateOfInformation { shift->_elem('DateOfInformation', @_); }
sub Domain { shift->_elem('Domain', @_); }
sub Keyword { shift->_elem('Keyword', @_); }
sub Language { shift->_elem('Language', @_); }
sub Restrictions { shift->_elem('Restrictions', @_); }
sub SecurityClassification { shift->_elem('SecurityClassification', @_); }
sub TargetEnvironment { shift->_elem('TargetEnvironment', @_); }
sub Version { shift->_elem('Version', @_); }
sub VersionDate { shift->_elem('VersionDate', @_); }
sub IsComposedOf { shift->_elem('IsComposedOf', @_); }
sub WasCreatedBy { shift->_elem('WasCreatedBy', @_); }
sub IsLocatedIn { shift->_elem('IsLocatedIn', @_); }
sub IsMadeOf { shift->_elem('IsMadeOf', @_); }
sub UniqueID { shift->_elem('UniqueID', @_); }
sub Comments { shift->_elem('Comments', @_); }
sub Reference { shift->_elem('Reference', @_); }
sub Signature { shift->_elem('Signature', @_); }
sub TitleLine { shift->_elem('TitleLine', @_); }
sub WebPage { shift->_elem('WebPage', @_); }
sub ContactIs { shift->_elem('ContactIs', @_); }
sub RecordMaintainer { shift->_elem('RecordMaintainer', @_); }

@Attributes = (
	'Abstract',
	'Cost',
	'DateOfInformation',
	'Domain',
	'Keyword',
	'Language',
	'Restrictions',
	'SecurityClassification',
	'TargetEnvironment',
	'Version',
	'VersionDate',
	'Comments',
	'Reference',
	'Signature',
	'TitleLine',
	'WebPage',
);

@OptionalAttributes = (
	'Keyword',
	'Restrictions',
	'TargetEnvironment',
	'Version',
	'VersionDate',
	'Comments',
	'Reference',
	'Signature',
	'WebPage',
);

@RequiredAttributes = (
	'Abstract',
	'Cost',
	'DateOfInformation',
	'Domain',
	'Language',
	'SecurityClassification',
	'TitleLine',
);

@Relationships = (
	'IsComposedOf',
	'WasCreatedBy',
	'IsLocatedIn',
	'IsMadeOf',
	'UniqueID',
	'ContactIs',
	'RecordMaintainer',
);

1;
