package BIDM::Tag::Link;

=head1 NAME

BIDM::Tag::Link

=head1 DESCRIPTION

BIDM Link tag.

=cut

use strict;

use vars qw(@ISA);

@ISA = qw(BIDM::Tag);

sub new
{
    my($class, $attributes) = @_;
    my($self, $k, $v);

    $self = new BIDM::Tag('link', $attributes);
    bless($self, $class);

    while (($k, $v) = each(%$attributes))
    {
	$self->$k($v);
    }

    return $self;
}

sub parse
{
    my($self, $obj) = @_;
    my(@comps, $type, $name, $model, $class_name);

    if (defined($name = $self->rel()))
    {
	$type = 'rel';
    }
    elsif (defined($name = $self->rev()))
    {
	$type = 'rev'
    }
    else
    {
	die "Invalid link";
    }
    
    @comps = split(/\./, $name);

    $model = shift(@comps);
    $class_name = shift(@comps);

    #
    # If we don't have an object instance yet, create one.
    # This happens when we find the first tag in a page.
    #

    if (!defined($obj))
    {
	$obj = BIDM::RIGObject::object_for_class($class_name);
    }

    #
    # See if we have an attribute or relationship.
    #

    my $cname;
    my $attr_obj = eval { $obj->lookup_attribute($comps[0]); };
    if ($attr_obj)
    {
	$cname = $attr_obj->name();
#	print "Link found attribute $cname\n";

	return $self->parse_attribute($obj, $attr_obj, @comps);
    }
    else
    {
#	print "fail: $@\n";
	my $rel_obj = eval { $obj->lookup_relationship($comps[0]); };
	if ($rel_obj)
	{
	    $cname = $rel_obj->name();
#	    print "Link found relationship $cname\n";
	    if ($type eq 'rel')
	    {
		return $self->parse_relationship($obj, $rel_obj, @comps)
	    }
	    else
	    {
		return $self->parse_inverse_relationship($obj, $rel_obj, @comps)
	    }
	}
	else
	{
#	    print "fail: $@\n";
	    print "Invalid attribute/relationship $comps[0]\n";
	    return $obj;
	}
    }
}

sub parse_attribute
{
    my($self, $obj, $attr_obj, @comps) = @_;
    my($cname);

    $cname = $attr_obj->name();
#    print "parse_attribute obj=$obj attr_obj=$attr_obj $cname @comps ", $self->href(), "\n";
    $obj->handle_tag_attribute($cname, $self);
    return $obj;
}

sub parse_relationship
{
    my($self, $obj, $rel_obj, @comps) = @_;
    my($cname);

    $cname = $rel_obj->name();
#    print "parse_relationship $cname @comps ", $self->href(), "\n";
    $obj->handle_tag_relationship($cname, $self);
    return $obj;
}

sub parse_inverse_relationship
{
    my($self, $obj, $rel_obj, @comps) = @_;
    my($cname);

    $cname = $rel_obj->name();
#    print "parse_inverse_relationship $cname @comps ", $self->href(), "\n";
    return $obj;
}

# sub old
# {
#     if (@comps == 3)
#     {
# 	#
# 	# Simple version: model.class.attribute
# 	#
# 	my ($model, $class, $attribute) = @comps;

# 	return $self->parse_link_attribute($obj, $type, $model,
# 					   $class, $attribute);
#     }
#     elsif (@comps == 4)
#     {
# 	#
# 	# Simple version: model.class.relationship.rel_class
# 	#
# 	my ($model, $class, $relationship, $rel_class) = @comps;

# 	return $self->parse_link_relationship($obj, $type, $model,
# 					      $class, $relationship,
# 					      $rel_class);
#     }
#     else
#     {
# 	die "Invalid link name $name\n";
#     }
#}

sub parse_link_attribute
{
    my($self, $obj, $type, $model, $class, $attribute) = @_;

    if (!defined($obj))
    {
	#
	# This must be the first tag. Have RIGObject create us an
	# object for this class.
	#
	$obj = BIDM::RIGObject::object_for_class($class);
    }

    $obj->attribute_link($type, $attribute, $self);

    return $obj;
}

sub parse_link_relationship
{
    my($self, $obj, $type, $model, $class, $relationship, $rel_class) = @_;

    if (!defined($obj))
    {
	#
	# This must be the first tag. Have RIGObject create us an
	# object for this class.
	#
	$obj = BIDM::RIGObject::object_for_class($class);
    }

    $obj->relationship_link($type, $relationship, $rel_class, $self);

    return $obj;
}

sub rel { shift->_elem('rel', @_); }
sub rev { shift->_elem('rev', @_); }
sub href { shift->_elem('href', @_); }
sub urn { shift->_elem('urn', @_); }
sub title { shift->_elem('title', @_); }

1;
